<?php

  /*

    Copyright (C) 2013-2014 Artem Prilutskiy, R3ABM (r3abm@dstar.su)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

  */

  define("REMOTE_CONTROL_SEMAPHORE", 1256);

  class RemoteControl
  {

    private $reconnects = array("Never", "Fixed", "5", "10", "15", "20", "25", "30", "60", "90", "120", "180");
    private $protocols = array("DExtra", "DPlus", "DCS", "CCS");
    private $directions = array("In", "Out");

    private $lock;
    private $handle;

    public function __construct($host, $port)
    {
      $this->lock = sem_get(REMOTE_CONTROL_SEMAPHORE, 1);
      sem_acquire($this->lock);

      $this->handle = fsockopen("udp://$host", $port);
    }

    public function __destruct()
    {
      fclose($this->handle);
      sem_release($this->lock);
    }

    public function logIn($password)
    {
      fwrite($this->handle, "LIN");

      $data = unpack("A3sign/Vsalt", fread($this->handle, 8));
      if ($data["sign"] != "RND")
        return FALSE;

      $hash = hash("sha256", pack("V", $data["salt"]) . $password, true);
      fwrite($this->handle, "SHA$hash");

      $data = unpack("A3sign/A*message", fread($this->handle, 80));
      if ($data["sign"] != "ACK")
        return $data["message"];

      return TRUE;
    }

    public function logOut()
    {
      fwrite($this->handle, "LOG");
    }

    public function queryCalls()
    {
      fwrite($this->handle, "GCS");

      $data = unpack("A3sign/A*records", fread($this->handle, 200));
      if ($data["sign"] != "CAL")
        return FALSE;

      $outcome = array();

      foreach (str_split($data["records"], 9) as $record)
        $outcome[] = unpack("A1type/A8call", $record);

      return $outcome;
    }

    function queryRepeater($call)
    {
      fwrite($this->handle, pack("A3A8", "GRP", $call));

      $buffer = fread($this->handle, 1024);

      $data = unpack("A3sign/A8call/Vreconnect/A8reflector", $buffer);
      if ($data["sign"] != "RPT")
        return FALSE;

      $outcome = array(
        "reconnect" => $this->reconnects[$data["reconnect"]],
        "reflector" => $data["reflector"],
        "links" => array()
      );
      if (strlen($buffer) > 23)
        foreach (str_split(substr($buffer, 23), 24) as $link)
        {
          $data = unpack("A8call/Vprotocol/Vlinked/Vdirection/Vdongle", $link);
          $outcome["links"][$data["call"]] = array(
            "protocol" => $this->protocols[$data["protocol"]],
            "direction" => $this->directions[$data["direction"]],
            "linked" => ($data["linked"] != 0),
            "dongle" => ($data["dongle"] != 0)
          );
        }

      return $outcome;
    }

    public function queryStarNetGroup($call)
    {
      // This function not tested

      fwrite($this->handle, pack("A3A8", "GSN", $call));

      $buffer = fread($this->handle, 1024);

      $data = unpack("A3sign/A8call/A8logoff/Vtimer/Vtimeout", $buffer);
      if ($data["sign"] != "SNT")
        return FALSE;

      $outcome = array(
        "logoff" => $data["logoff"],
        "timer" => $data["timer"],
        "timeout" => $data["timeout"],
        "users" => array()
      );

      if (strlen($message) > 27)
        foreach (str_split(substr($buffer, 27), 16) as $user)
        {
          $data = unpack("A8call/Vtimer/Vtimeout", $user);
          $outcome["users"][$data["call"]] = array(
            "timer" => $data["timer"],
            "timeout" => $data["timeout"]
          );
        }

      return $outcome;
    }

    public function linkRepeater($call, $reconnect, $reflector)
    {
      $index = array_search($reconnect, $this->reconnects);
      if ($index === FALSE)
        return FALSE;

      fwrite($this->handle, pack("A3A8VA8", "LNK", $call, $index, $reflector));

      $data = unpack("A3sign/A*message", fread($this->handle, 80));
      if ($data["sign"] != "ACK")
        return $data["message"];

      return TRUE;
    }

    public function unlinkRepeater($call, $protocol, $reflector)
    {
      $index = array_search($protocol, $this->protocols);
      if ($index === FALSE)
        return FALSE;

      fwrite($this->handle, pack("A3A8VA8", "UNL", $call, $index, $reflector));

      $data = unpack("A3sign/A*message", fread($this->handle, 80));
      if ($data["sign"] != "ACK")
        return $data["message"];

      return TRUE;
    }

    public function dropUser($call, $name)
    {
      fwrite($this->handle, pack("A3A8A8", "LGO", $call, $name));

      $data = unpack("A3sign/A*message", fread($this->handle, 80));
      if ($data["sign"] != "ACK")
        return $data["message"];

      return TRUE;
    }

  }

?>